#-*- coding: utf-8 -*-

# Copyright 2010 Mir Calculate Ltd. http://www.calculate-linux.org
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
from cl_desktop import desktop, __app__, __version__
from cl_opt import opt, TitledHelpFormatter
import sys
from cl_share_cmd import share_cmd

# Перевод сообщений для программы
from cl_lang import lang
lang().setLanguage(sys.modules[__name__])

# Использование программы
USAGE = _("%prog [options] user")

# Коментарии к использованию программы
COMMENT_EXAMPLES = _("Create home directory for the user_name")

# Пример использования программы
EXAMPLES = _("%prog user_name")

# Описание программы (что делает программа)
DESCRIPTION = _("Create home directory for the new user account")

# Опции командной строки
CMD_OPTIONS = [{'longOption':"progress",
                'help':_("show progress bar for xdm startup")}]

class createhome(share_cmd):
    def __init__(self):
        # Объект опций командной строки
        self.optobj = opt(\
            package=__app__,
            version=__version__,
            usage=USAGE,
            examples=EXAMPLES,
            comment_examples=COMMENT_EXAMPLES,
            description=DESCRIPTION,
            option_list=CMD_OPTIONS + opt.variable_control+opt.color_control,
            formatter=TitledHelpFormatter(),
            check_values=self.checkOpts)
        # Создаем объект логики
        self.logicObj = desktop()
        # Создаем переменные
        self.logicObj.createClVars()

    def checkOpts(self, optObj, args):
        """Проверка опций командной строки"""
        if not args:
            errMsg = _("no such argument") + ":" + " %s" %USAGE.split(" ")[-1]
            self.optobj.error(errMsg)
            return False
        if len(args)>1:
            errMsg = _("incorrect argument") + ":" + " %s" %" ".join(args)
            self.optobj.error(errMsg)
            return False
        userName = args[0]
        if userName == "root":
            errMsg = _("invalid username") + ":" + " %s" %" ".join(args)
            self.optobj.error(errMsg)
            return False
        # Проверка на существование пользователя
        if not self.logicObj.existsUser(userName):
            return False
        return optObj, args

    def setUserName(self, userName):
        """Установка имени пользователя"""
        self.logicObj.clVars.Set("ur_login", userName, True)

    def createHome(self, optObj):
        """Создание домашней директории"""
        return self.logicObj.createHome(optObj.progress)